#!/usr/bin/perl -w

use strict;

use File::Spec;
use File::Copy;
use Data::Dumper;

sub findfiles($) {
	my $dir = shift;

	my @dirs = ($dir);
	my @files = ();

	local *DIR;

	while (@dirs) {
		$dir = shift @dirs;
		opendir(*DIR, "$dir") or die("$dir: $!\n");
		while (my $basename = readdir(*DIR)) {
			if ($basename ne File::Spec->curdir() && $basename ne File::Spec->updir()) {
				my $fullname = File::Spec->catfile("$dir", "$basename");
				if (-d "$fullname") {
					push @dirs, "$fullname";
				}
				elsif ($basename =~ /.pro$/) {
					push @files, "$fullname";
				}
			}
		}
		closedir(*DIR);
	}

	return @files;
}

sub setvalue($$$;$) {
	my $filename = shift;
	my $varname = shift;
	my $value = shift;
	my $operator = shift || '+=';

	local *FILE;

	my @lines = ();

	open(*FILE, "< $filename") or die("$filename: $!\n");
	while (my $line = <FILE>) {
		push @lines, $line;
	}
	close(*FILE);

	push @lines, "$varname $operator $value\n";

	open(*FILE, "> $filename") or die("$filename: $!\n");
	foreach my $line (@lines) {
		print FILE "$line";
	}
	close(*FILE);
}

my $ARCH = lc($ARGV[0]);
my $DIR = $ARGV[1];
my $ACTION = $ARGV[2];

if ($ACTION eq 'apply') {

	my @files = findfiles("$DIR");

	my $CXXFLAGS = '';
	my $LDFLAGS = '';
	my $CPP = '';
	my $CXX = '';
	my $LINK = '';

	if ($ARCH eq 'ia64' || $ARCH eq 'amd64') {
		$CXXFLAGS = '-m64 -fPIC';
		$LDFLAGS = '-m64';
	}
	elsif ($ARCH eq 'i386' || $ARCH eq 'i586') {
		my $LDLIBPATH = $ENV{'LD_LIBRARY_PATH'} || '';
		my @LDLIBPATHS = split(/:/,$LDLIBPATH);
		$CXXFLAGS = '-I/usr/share/qt3/mkspecs/linux-g++-32 -m32 -fPIC';
		my @ld_flags = ();
		my @ld_paths = ();
		foreach my $ldp (@LDLIBPATHS) {
			if ($ldp && -d "$ldp") {
				push @ld_flags, "-L$ldp";
				push @ld_paths, "$ldp";
			}
		}
		$LDFLAGS = join(' ', @ld_flags).' -m32 -lstdc++';
		$CPP = 'LD_LIBRARY_PATH="'.join(':',@ld_paths).':$$LD_LIBRARY_PATH" gcc';
		$CXX = 'LD_LIBRARY_PATH="'.join(':',@ld_paths).':$$LD_LIBRARY_PATH" g++';
		$LINK = 'LD_LIBRARY_PATH="'.join(':',@ld_paths).':$$LD_LIBRARY_PATH" g++';
	}
	else {
		die("unsupported architecture: $ARCH\n");
	}

	foreach my $file (@files) {
		print "\tpatching $file\n";
		if ($CXXFLAGS) {
			setvalue("$file", "QMAKE_CXXFLAGS", "$CXXFLAGS");
		}
		if ($LDFLAGS) {
			setvalue("$file", "QMAKE_LFLAGS", "$LDFLAGS");
		}
		if ($CPP) {
			setvalue("$file", "QMAKE_CC", "$CPP", '=');
		}
		if ($CXX) {
			setvalue("$file", "QMAKE_CXX", "$CXX", '=');
		}
		if ($LINK) {
			setvalue("$file", "QMAKE_LINK", "$LINK", '=');
		}
	}

	my $shlibfile = '/srv/arakhne.org/cross32/lib/shlibs.local.'.$ARCH;

	if ( -f "$shlibfile") {
		my $targetFile = File::Spec->catfile(".","debian","shlibs.local");
		copy("$shlibfile", "$targetFile") or die("$shlibfile: $!\n");
	}

}
elsif ($ACTION eq 'clean') {
	my $shlibfile = File::Spec->catfile(".","debian","shlibs.local");
	unlink("$shlibfile");
}

exit(0);
__END__
